/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Audrius Urmanavicius
 *
 * File:
 *    customizer.c
 *
 * Description:
 *    Implements device cusstomizing routines, like menu colors etc.
 *
 *****************************************************************************/
#include "configAppl.h"
#ifdef INCLUDE_CUSTOMIZER
/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include <string.h>
#include "menu.h"
#include "lcd.h"
#include "key.h"
#include "color_editor.h"
#include "arrow_img.h"
#include "customizer.h"
#include "nvfs.h"
#include "arrow_img.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
/*
#define ARROW_LEFT_BLANK	0x01
#define ARROW_LEFT_OPAQUE	0x03
#define ARROW_LEFT_TRANSP	0x07

#define ARROW_RIGHT_BLANK	0x08
#define ARROW_RIGHT_OPAQUE	0x18
#define ARROW_RIGHT_TRANSP	0x38


#define COLOR_MAX			7
#define BAR_STEP			14
#define BAR_LEFT			16
*/
/******************************************************************************
 * Global variables
 *****************************************************************************/
static const tU32 MAGIC_VALUE = 0x646F6F47; // ASCII "Good" :)
static tU8 contrast = 56;
static const char * fName = "Custom";

/******************************************************************************
 * Local function forward definitions
 *****************************************************************************/
static void setDefaults(tU32 setAll);
static void setContrast(tU32 dummy);
static void setColors(tU32 dummy);
static void setColor(tU32 idx);
static void loadSettings(tU32 dummy);
static void saveSettings(tU32 dummy);

/******************************************************************************
 * Menu definitions
 *****************************************************************************/
static tMenuItem const items[] = {
	{"Contrast", setContrast, NULL},
	{"Menu colors", setColors, NULL},
	{"Set defaults", setDefaults, 1},
	{"Save settings", saveSettings, NULL},
	{"Load settings", loadSettings, NULL},
	{"Exit", NULL, NULL} };

static tMenu menu = {"Customizer", NULL, items, sizeof(items) / sizeof(tMenuItem), 0};


static tMenuItem const colorItems[] = {
	{"Border", 		setColor, 0},
	{"Caption Bg", 	setColor, 1},
	{"Caption Fg", 	setColor, 2},
	{"Item Bg", 	setColor, 4},
	{"Item Fg", 	setColor, 5},
	{"Cursor frame", setColor, 3},
	{"Cursor Bg", 	setColor, 6},
	{"Cursor Fg", 	setColor, 7},
	{"Set defaults", setDefaults, 0},
	{"Exit", NULL, NULL} };

static tMenu colorsMenu = {"MENU COLORS", NULL, colorItems, sizeof(colorItems) / sizeof(tMenuItem), 0};

/*****************************************************************************
 *
 * Description:
 *    Saves customized settings to NVFS
 *
 ****************************************************************************/
static void saveSettings(tU32 dummy) {
	tU32 magic = MAGIC_VALUE;
	FILE *f = fopen(fName, "w");
	fwrite(&magic, sizeof(magic), 1, f);
	fwrite(&defMenuColors, sizeof(defMenuColors), 1, f);
	fwrite(&contrast, sizeof(contrast), 1, f);
	fclose(f);
}
/*****************************************************************************
 *
 * Description:
 *    Sets default colors and contrast.
 *
 ****************************************************************************/
static void loadSettings(tU32 dummy) {
	tU32 magic;
	FILE *f = fopen(fName, "r");
	fread(&magic, sizeof(magic), 1, f);
	if(magic == MAGIC_VALUE) {
		fread(&defMenuColors, sizeof(defMenuColors), 1, f);
		fread(&contrast, sizeof(contrast), 1, f);
		lcdContrast(contrast);
	}
	fclose(f);

	if(defMenuColors.itemBg == defMenuColors.itemFg)
		setDefaults(1);
}

/*****************************************************************************
 *
 * Description:
 *    Sets default colors and contrast.
 *
 ****************************************************************************/
void setDefaults(tU32 setAll) {
	defMenuColors = (tMenuColors){0xFF, 0x33, 0x48, 0xE0, 0x24, 0x91, 0x03, 0xF8};
	if(setAll) {
		contrast = 56;
		lcdContrast(contrast);
	}
};

/*****************************************************************************
 *
 * Description:
 *    Adjusts selected menu color.
 *
 * Parameters:
 *    idx: Index of color to be adjusted
 *
 ****************************************************************************/
void setColor(tU32 idx) {
	editColor((tU8*)&defMenuColors + idx, colorItems[idx].caption);
};

/*****************************************************************************
 *
 * Description:
 *    Adjusts LCD contrast.
 *
 ****************************************************************************/
#define MAX_CONTRAST	78
#define MIN_CONTRAST	28
#define BAR_STEP		2
#define BAR_WIDTH		((MAX_CONTRAST - MIN_CONTRAST) * BAR_STEP)
#define BAR_LEFT		(65 - BAR_WIDTH / 2)

static void drawBar(tU8 value) {
	lcdRect(BAR_LEFT, 38, BAR_STEP * value, 8, clGray);			
	lcdRect(BAR_LEFT + BAR_STEP * value, 38, BAR_WIDTH - BAR_STEP * value, 8, clBlack);			
}

void setContrast(tU32 dummy) {
	tU8 key, i, col;
	lcdColor(clBlack, clBlack);
	lcdClrscr();
	lcdRect(0, 0, 130, 20, defMenuColors.captionBg);
	lcdColor(defMenuColors.captionBg, defMenuColors.captionFg);
	lcdGotoxy(65 - 8 * 4, 2);
	lcdPuts("CONTRAST");

	lcdRect(BAR_LEFT - 1, 37, BAR_WIDTH + 2, 10, clWhite);
	drawBar(contrast - MIN_CONTRAST);
	lcdIcon(BAR_LEFT - 13, 33, 10, 18, TRUE, 0x01, (contrast > MIN_CONTRAST)? ImgLeftOpaque+4: ImgLeftTransparent+4);
	lcdIcon(BAR_LEFT + BAR_WIDTH + 3, 33, 10, 18, TRUE, 0x01, (contrast < MAX_CONTRAST)? ImgRightOpaque+4: ImgRightTransparent+4);

	//Draw color bands
	for(i = 0; i < 8; i++) {
		col = (i << 5) | (i << 2) | (i >> 1);
		lcdRect(1 + i * 16, 64, 16, 16, col & 0xE0);
		lcdRect(1 + i * 16, 80, 16, 16, col & 0x1C);
		lcdRect(1 + i * 16, 96, 16, 16, col & 0x03);
		lcdRect(1 + i * 16, 112, 16, 16, col);
	}

	do {
		key = getKey();
		if(key & (KEY_LEFT | KEY_RIGHT)) {
			if(key == KEY_LEFT && contrast > MIN_CONTRAST) {
				if(contrast == MAX_CONTRAST)
					lcdIcon(BAR_LEFT + BAR_WIDTH + 3, 33, 10, 18, TRUE, 0x01, ImgRightOpaque+4);
				contrast--;
				if(contrast == MIN_CONTRAST)
					lcdIcon(BAR_LEFT - 13, 33, 10, 18, TRUE, 0x01, ImgLeftTransparent+4);
			} else if (key == KEY_RIGHT && contrast < MAX_CONTRAST) {
				if(contrast == MIN_CONTRAST)
					lcdIcon(BAR_LEFT - 13, 33, 10, 18, TRUE, 0x01, ImgLeftOpaque+4);
				contrast++;
				if(contrast == MAX_CONTRAST)
					lcdIcon(BAR_LEFT + BAR_WIDTH + 3, 33, 10, 18, TRUE, 0x01, ImgRightTransparent+4);
			}
			drawBar(contrast - MIN_CONTRAST);
			lcdContrast(contrast);
		}
		
	} while(key != KEY_CENTER);
};

/*****************************************************************************
 *
 * Description:
 *    Invokes color setup menu
 *
 ****************************************************************************/
void setColors(tU32 dummy) {
	tU8 i;
	lcdColor(clBlack, clBlack);
	lcdClrscr();
    colorsMenu.initialChoice = 0;
    do {
        i = runMenu(&colorsMenu);
    } while(i < colorsMenu.itemCount - 1);
};

/*****************************************************************************
 *
 * Description:
 *    Invokes Customizer menu
 *
 ****************************************************************************/
void runCustomizer(tU32 param) {
	tU8 i;

	if(param == CUSTOMIZER_LOAD_SETTINGS) {
		loadSettings(0);
		return;
	}
	if(param == CUSTOMIZER_SAVE_SETTINGS) {
		saveSettings(0);
		return;
	}
	if(param == CUSTOMIZER_RESET_SETTINGS) {
		setDefaults(1);
		return;
	}

    menu.initialChoice = 0;
	lcdColor(clBlack, clBlack);
	lcdClrscr();
    do {
        i = runMenu(&menu);
    } while(i < menu.itemCount - 1);
}

#endif /*INCLUDE_CUSTOMIZER */
