/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Embedded Artists AB
 *
 * File:
 *    key.c
 *
 * Description:
 *    Implements sampling and handling of joystick key.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include "key.h"
#include "hw.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define FIRST_REPEAT  4
#define SECOND_REPEAT 3

#define KEYPROC_STACK_SIZE 300
#define KEY_QUEUE_SIZE  16

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static tQueue keyQueue;
static void *keyQueueArea[KEY_QUEUE_SIZE];
static tU8 keyRepeat;
static tTimer keyTimer;

static volatile tU8 activeKey = KEY_NOTHING;

/*****************************************************************************
 *
 * Description:
 *    Function waits for keypress and returns it's code [getch() equivalent]
 *
 ****************************************************************************/
tU8
getKey(void) {
    tU8 error;
    return (tU8)(tU32) osPendQueue(&keyQueue, 0, &error);
}


/*****************************************************************************
 *
 * Description:
 *    Function to check if any key press has been detected
 *
 ****************************************************************************/
tU8
checkKey(void) {
    tU8 error;
    return (tU8)(tU32) osAcceptQueue(&keyQueue, &error);
}

/*****************************************************************************
 *
 * Description:
 *    Function to check current (instantaneous) key state
 *
 ****************************************************************************/
tU8
checkKey2(void) {
  return activeKey;
}

/*****************************************************************************
 *
 * Description:
 *    Sample key states
 *
 ****************************************************************************/
void
sampleKey(void) {
	tU8 readKeys, error;

    static tU8 lastKeys = 0;

  //get sample
	readKeys = getKeys();

    if(readKeys) {
        if(lastKeys & readKeys) { //Autorepeat
            keyRepeat++;
            if(keyRepeat == FIRST_REPEAT)
                keyRepeat = 0;
        }
        if(keyRepeat == 0) {
            osPostQueue(&keyQueue, (void *)readKeys, &error);
            activeKey = readKeys;
        }
    } else {
        activeKey = KEY_NOTHING;
        keyRepeat = 0;
    }
    lastKeys = readKeys;
}

/*****************************************************************************
 *
 * Description:
 *    Creates and starts the key sampling process.
 *
 ****************************************************************************/
void
initKeyProc(void) {
	osCreateQueue(&keyQueue, keyQueueArea, KEY_QUEUE_SIZE);
	osCreateTimer(&keyTimer, sampleKey, TRUE, 5); //Sample keys each 50ms
}

