#ifndef _NVFS_H_
#define _NVFS_H_

//Non-Volatile File System (NVFS) header file

#include "configAppl.h"
#ifdef INCLUDE_NVFS

#define FOPEN_MAX	5

#define FILE_CLOSED		0x00
#define FILE_OPEN		0x01

#ifndef SEEK_SET
	#define SEEK_SET	0
#endif
#ifndef SEEK_CUR
	#define SEEK_CUR	1
#endif
#ifndef SEEK_END
	#define SEEK_END	2
#endif


typedef enum {fsSuccess = 0, fsInvalidArgument, fsInvalidAddress, fsNotOpen, fsNotFound, fsInvalidName, fsNoMoreSpace,
				fsEOF, fsFilesystemError, fsEepromError, fsBadSeek, fsTooManyOpenFiles} fsErrno;
typedef tU8 SectAddr;

typedef struct {
	tU16 fpos;				// current write/read position offset into file in bytes
	tU16 metadataOffset;	// Offset of this file's metadata into metaFile
	SectAddr firstSect;		// first data sector
	SectAddr currentSect;	// Address of sector which contains fpos
	tBool EOF;				// End Of File flag
	tU8 state;				// File state
	fsErrno errno;			// Last error code
} fsFile;


extern fsErrno _fsErrno;

fsFile* fsOpen(const char *name, const char *mode);
int fsClose(fsFile *file);
int fsRename(const char *oldfname, const char *newfname);
int fsRemove(const char *fname);
size_t fsRead(void *buffer, const size_t size, const size_t num, fsFile *file);
size_t fsWrite(const void *buffer, const size_t size, const size_t num, fsFile *file);
int fsSeek(fsFile *file, int pos, int origin);
int fsTruncate(fsFile *file);
size_t fsFileSize(fsFile *file);
int fsInit(void);
int fsFormat(void);
fsErrno fsGetLastError(void);

#ifdef STD_IO_NAMING

#define FILE		fsFile
#define fopen		fsOpen
#define fclose		fsClose
#define rename		fsRename
#define remove	 	fsRemove
#define fread		fsRead
#define fwrite		fsWrite
#define fseek		fsSeek
#define ftrunc		fsTruncate

#define fflush(f)	(void *)0
#define feof(f)		((f)->EOF)
#define ferror(f) 	((f)->errno)

#endif /* STD_IO_NAMING */

#endif /* INCLUDE_NVFS */

#endif /*_NVFS_H_ */
